@@ -22,34 +22,15 @@
 #
 
 import binascii
-from enum import Enum, IntEnum
-
-from py7zr.exceptions import InternalError
+from enum import Enum
+from typing import Optional
 
 MAGIC_7Z = binascii.unhexlify('377abcaf271c')
+FINISH_7Z = binascii.unhexlify('377abcaf271d')
+READ_BLOCKSIZE = 32248
+QUEUELEN = READ_BLOCKSIZE * 2
 
-
-class Configuration:
-    '''Singleton global configuration holder'''
-
-    P7ZIP_MAJOR_VERSION = b'\x00'
-    P7ZIP_MINOR_VERSION = b'\x04'
-    read_blocksize = 32248
-    _instance = None
-
-    @classmethod
-    def get(cls, key: str) -> int:
-        if cls._instance is None:
-            cls._instance = cls()
-        return getattr(cls._instance, key)
-
-    @classmethod
-    def set(cls, size):
-        if cls._instance is None:
-            cls._instance = cls()
-            setattr(cls._instance, 'read_blocksize', size)
-        else:
-            raise InternalError("Try to set buffer size after instanced.")
+READ_BLOCKSIZE = 32248
 
 
 class ByteEnum(bytes, Enum):
@@ -57,6 +38,7 @@ class ByteEnum(bytes, Enum):
 
 
 class Property(ByteEnum):
+    """Hold 7zip property fixed values."""
     END = binascii.unhexlify('00')
     HEADER = binascii.unhexlify('01')
     ARCHIVE_PROPERTIES = binascii.unhexlify('02')
@@ -86,6 +68,7 @@ class Property(ByteEnum):
 
 
 class CompressionMethod(ByteEnum):
+    """Hold fixed values for method parameter."""
     COPY = binascii.unhexlify('00')
     DELTA = binascii.unhexlify('03')
     BCJ = binascii.unhexlify('04')
@@ -94,6 +77,9 @@ class CompressionMethod(ByteEnum):
     ARM = binascii.unhexlify('07')
     ARMT = binascii.unhexlify('08')
     SPARC = binascii.unhexlify('09')
+    # SWAP = 02..
+    SWAP2 = binascii.unhexlify('020302')
+    SWAP4 = binascii.unhexlify('020304')
     # 7Z = 03..
     LZMA = binascii.unhexlify('030101')
     PPMD = binascii.unhexlify('030401')
@@ -114,30 +100,20 @@ class CompressionMethod(ByteEnum):
     MISC_LZH = binascii.unhexlify('0406')
     NSIS_DEFLATE = binascii.unhexlify('040901')
     NSIS_BZIP2 = binascii.unhexlify('040902')
+    #
+    MISC_ZSTD = binascii.unhexlify('04f71101')
+    MISC_BROTLI = binascii.unhexlify('04f71102')
+    MISC_LZ4 = binascii.unhexlify('04f71104')
+    MISC_LZS = binascii.unhexlify('04f71105')
+    MISC_LIZARD = binascii.unhexlify('04f71106')
     # CRYPTO 06..
     CRYPT_ZIPCRYPT = binascii.unhexlify('06f10101')
     CRYPT_RAR29AES = binascii.unhexlify('06f10303')
     CRYPT_AES256_SHA256 = binascii.unhexlify('06f10701')
 
 
-class FileAttribute(IntEnum):
-    DIRECTORY = 0x10
-    READONLY = 0x01
-    HIDDEN = 0x02
-    SYSTEM = 0x04
-    ARCHIVE = 0x20
-    DEVICE = 0x40
-    NORMAL = 0x80
-    TEMPORARY = 0x100
-    SPARSE_FILE = 0x200
-    REPARSE_POINT = 0x400
-    COMPRESSED = 0x800
-    OFFLINE = 0x1000
-    ENCRYPTED = 0x4000
-    UNIX_EXTENSION = 0x8000
-
-
 class SupportedMethods:
+    """Hold list of methods which python3 can support."""
     formats = [{'name': "7z", 'magic': MAGIC_7Z}]
     codecs = [{'id': CompressionMethod.LZMA, 'name': "LZMA"},
               {'id': CompressionMethod.LZMA2, 'name': "LZMA2"},
@@ -149,3 +125,31 @@ class SupportedMethods:
               {'id': CompressionMethod.BCJ_ARMT, 'name': "ARMT"},
               {'id': CompressionMethod.BCJ_SPARC, 'name': 'SPARC'}
               ]
+
+
+# this class is Borg/Singleton
+class ArchivePassword:
+
+    _shared_state = {
+        '_password': None,
+    }
+
+    def __init__(self, password: Optional[str] = None):
+        self.__dict__ = self._shared_state
+        if password is not None:
+            self._password = password
+
+    def set(self, password):
+        self._password = password
+
+    def get(self):
+        if self._password is not None:
+            return self._password
+        else:
+            return ''
+
+    def __str__(self):
+        if self._password is not None:
+            return self._password
+        else:
+            return ''
